import {
	Injectable,
	InternalServerErrorException,
	NotFoundException,
} from "@nestjs/common";
import { DbService } from "apps/backend/src/app/db/db.service";
import { FilesGroupTable, FilesTable } from "apps/backend/src/app/db/schema";
import { and, eq, ilike } from "drizzle-orm";

@Injectable()
export class GroupsService {
	constructor(private readonly database: DbService) {}

	async getGroupsByName(limit: number, offset: number, search: string) {
		return await this.database
			.use()
			.select()
			.from(FilesGroupTable)
			.where(ilike(FilesGroupTable.groupName, search))
			.limit(limit)
			.offset(offset)
			.prepare("getGroupsByName")
			.execute();
	}

	async newGroup(groupName: string) {
		return await this.database
			.use()
			.insert(FilesGroupTable)
			.values({
				groupName: groupName,
			})
			.returning()
			.prepare("newGroup")
			.execute();
	}

	async deleteGroup(groupId: string) {
		const groupInDb = await this.database
			.use()
			.select()
			.from(FilesGroupTable)
			.where(eq(FilesGroupTable.uuid, groupId))
			.prepare("findGroupById")
			.execute();
		if (groupInDb.length === 0) throw new NotFoundException("Group not found");
		// Replace entry by null
		await this.database
			.use()
			.update(FilesTable)
			// @ts-ignore
			.set({ groupId: null })
			.where(eq(FilesTable.groupId, groupId))
			.prepare("updateFilesGroupReference")
			.execute();
		try {
			await this.database
				.use()
				.delete(FilesGroupTable)
				.where(eq(FilesGroupTable.uuid, groupId));
			return true;
		} catch (e) {
			throw new InternalServerErrorException("Error while deleting group");
		}
	}

	async findFilesForGroup(
		limit: number,
		offset: number,
		searchField: string,
		groupId: string,
	) {
		return await this.database
			.use()
			.select()
			.from(FilesTable)
			.where(
				and(
					eq(FilesTable.groupId, groupId),
					ilike(FilesTable.fileName, String(`%${searchField}%`)),
				),
			)
			.limit(limit)
			.offset(offset)
			.prepare("findFilesInGroup")
			.execute();
	}
}
